/*
 * Пример работы с библиотекой I2C
 * Почти готовые функции:
 * - ScanDevice
 * - isReady
 */
#include "STM32.h"
#include "IO_Digital.h"
#include "I2C.h"
#include "UART.h"
#include "string.h"
#include "stdint.h"
#include "STM_BME280.h"

#define   ADDRESS_BME280 0x76

I2C         myI2C(I2C2, false);
uart        myUART(USART1, false);
S_BME280    myBME;

bool FlagBME = 0;

int main(void)
{
  int16_t Temp;
  uint8_t Counts = 0;
  uint8_t Buff[256] = {0,};
  char msg[128] = { 0, };

  SystemClock_Config(Quartz_8);
  myUART.init(256000);
  myI2C.Init(Mode_Fast);

  /*
   * Сканируем количество и адреса устройств
   */
  myUART.println("Scan Device : ");
  Counts = myI2C.ScanDevice(Buff);
  if(Counts)
  {
    for(uint8_t I=0; I<Counts; I++)
    {
      sprintf(msg, "Device: %d Address - 0x%02X OK \n\r", I, Buff[I]);
      myUART.print(msg);
    }
  }
  sprintf(msg, "\n\rFound - %d Device\n\r", Counts);
  myUART.println(msg);


  /*
   * Инициализируем BMP или BME
   */
  myUART.println("BMP or BME Init");
  Temp = myBME.Init(&myI2C);
  if(Temp < 0)
  {
    FlagBME = 0;
    sprintf(msg, "BME Read Error \n\r");
  }
  else if(Temp == 0x60)
  {
    sprintf(msg, "BME ID - 0x%02X OK \n\r", Temp);
    FlagBME = 1;
  }
  myUART.println(msg);


  while(1)
  {
      /*
       * Замеры BME 280
       */
    if(FlagBME)
    {
      myUART.println("--- Measuring BME ---\n\r");
      sprintf(msg, "Temperature : %.2f *C\n\r", myBME.readTemperature());
      myUART.print(msg);

      sprintf(msg, "Humidity    : %.2f %%\n\r", myBME.readHumidity());
      myUART.print(msg);

      float pressure = myBME.readPressure();                  // Читаем давление в [Па]
      sprintf(msg, "Pressure    : %.2f hPa\n\r", pressure / 100.0F);
      myUART.print(msg);

      sprintf(msg, "Pressure    : %.2f mm Hg\n\r", _pressureToMmHg(pressure));
      myUART.println(msg);
    }

    delay(5000);
  }
}


/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  __disable_irq();
  while (1)
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

