/*
 * Пример работы с базовыми таймерами TIM6, TIM7
 *
 * Основан на сдвиге таблицы векторов в ОЗУ
 *
 */
#include "STM32.h"
#include "constants.h"
//-----------------------------------
#include "IO_Digital.h"
#include "UART.h"
#include "string.h"
#include "stdint.h"
#include "TIM_Base.h"

// Здесь мы резервируем место под новую таблицу векторов прерывания
uint32_t ramVectorTable[constants::isrVectorTableSize] __attribute__(( aligned (constants::isrVectorTableAlingment) ));
extern uint32_t g_pfnVectors[constants::isrVectorTableSize];

// ------------------ Объявляем "Левые" обработчики прерывания -----------------//
void myTIM_ISR6(void);
void myTIM_ISR7(void);

// --------------- Классы написанные и рассмотренные ранее ---------------------//
uart        myUART(USART1, false);
//IO_Digital  Led(GPIOE, GPIO_PIN_0, Output, No_Pull);
IO_Digital  Led(GPIOA, GPIO_PIN_1, Output, No_Pull);

// -------- Здесь классы по новому принципу обработки прерываний ---------------//
TIM_Base    myTIM6(TIM6);                                             // Объявляем класс myTIM6
TIM_Base    myTIM7(TIM7);                                             // Объявляем класс myTIM7

int main(void)
{
  SystemClock_Config(Quartz_8);

  /*----------------------------------------------------------------------------
   * Перкидывание векторов прерывания в ОЗУ
   * Сюда пальцы не суём
   *--------------------------------------------------------------------------*/
  for (uint16_t i = 0; i < constants::isrVectorTableSize; i++)        // Переносим вектора прерывания из FLASH
    ramVectorTable[i] = g_pfnVectors[i];                              // в ОЗУ

  __disable_irq();                                                    // Запрещаем прерывания
    SCB->VTOR = (uint32_t)&ramVectorTable;                            // Устанавливаем смещение для новой таблицы векторов прерываний
  __DSB();                                                            // Синхронизация
  __enable_irq();                                                     // Разрешием прерывания

  PeripheralUnit::setIsrVectorTable(ramVectorTable);                  // Инициализируем класс векторов
  //----------------------------------------------------------------------------

  myUART.init(256000);                                      // Инициализируем UART, как было описано ранее

  // ------------ Здесь новьё. Инициализация таймеров по новой технологии ------/
  myTIM6.InitSec(3);                                        // Инициализируем таймер на 3 секунды
  myTIM6.SetOPM(true);                                      // Врубаем функцию OPM. Режим одного импульса
  myTIM6.Start();                                           // Запускаем таймер

  myTIM7.SetCallbackIRQ(myTIM_ISR7);                        // Устанавливаем свой Обработчик прерывания
  myTIM7.Init(8399, 19999);                                 // Тактирование частотой 10КГц. Прерывание каждые 2 сек.
  myTIM7.Start();                                           // Запускаем счётчик

  bool Togle = false;                                       // Флаг "триггер"
  while(1)                                                  // В цикле способ обработки прерываний через флаги
  {
    bool interruptOccured = myTIM6.getInterruptFlag();      // Считываем флаг
    if (interruptOccured == true)                           // Если флаг от TIM6 поднят,
    {
      Led.digitalTogglePin();                               // Переключаем светодиод,
      myTIM6.clearInterruptFlag();                          // Сбрасываем флаг от TIM6.

      if(Togle)                                             // Если триггер включён,
        myTIM6.InitSec(3);                                  // переинициализируем таймер на 3 секунды
      else                                                  // Если триггер выключен,
        myTIM6.InitSec(1);                                  // переинициализируем таймер на 1 секунду.

      Togle = !Togle;                                       // Переключаем триггер
      myTIM6.Start();                                       // Заново запускаем таймер.
    }
  }
}

/*------------------------------------------------------------------------------
 * Внешние функции прерывания, которые можно делать, а можно и нахрен послать,
 * если работать только по флагам.
 *----------------------------------------------------------------------------*/
void myTIM_ISR7(void)
{
  myUART.print("TIM7 Function Interrupt - ");
  myUART.println(millis() / 1000);
}

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  __disable_irq();
  while (1)
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

