
#include <STM32.h>
#include <IO_Digital.h>
#include <RTC_IT.h>

#define VECTORTABLE_SIZE (391)//256)          // Размер таблицы векторов ToDo разобраться почему такое число
#define VECTORTABLE_ALIGNMENT (0x100ul)       // Выравнивание таблицы ToDo разобраться почему именно 0x100

extern uint32_t g_pfnVectors[];               // Указатель на вектора, находится в ASM скрипте
// Объявляем глобальную переменную в начале ОЗУ, где будет новая таблица
volatile uint32_t vectorTable_RAM[VECTORTABLE_SIZE] __attribute__(( aligned (VECTORTABLE_ALIGNMENT) ));


IO_Digital  Led_01(GPIOA, GPIO_PIN_1, Output, No_Pull);
RTC_main    myRTC(RTC_LSE);

void setIRQ(void);

int main(void)
{
  for (uint32_t i = 0; i < VECTORTABLE_SIZE; i++) vectorTable_RAM[i] = g_pfnVectors[i];   // Копируем таблицу векторов на новое место в ОЗУ
  __disable_irq();                                                                        // Отключаем прерывания
  SCB->VTOR = (uint32_t)&vectorTable_RAM;                                                 // Указываем новое смещение таблицы векторов
  __DSB();                                                                                // Синхронизация
//  vectorTable_RAM[SysTick_IRQn + 16] = (uint32_t)SysTick_Handler_RAM;                       // Это просто для примера, как подменять вектор
  __enable_irq();                                                                         // Разрешаем прерывания
/*
 * Далее идёт то, что и должно быть.
 * Все операции находящиеся выше должны быть выполнены в первую очередь.
 */

  SystemClock_Config(Quartz_8);                                                           // Инициализация тактового генератора

  myRTC.Reinit();

  myRTC.setIT_WUCK(setIRQ, 1);                                                            // Внедрение в вектор своей функции

  while (1)
  {
    delay(1000);
  }
}

/*
 *  Внешний обработчик прерывания от таймера побудки
 */
void setIRQ(void)
{
  Led_01.digitalTogglePin();
}


/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  __disable_irq();
  while (1)
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
