/*
 * RTC.cpp
 *
 *  Created on: 26 окт. 2021 г.
 *      Author: User
 */

//****************************************************************************** 
//  Секция include: здесь подключается заголовочный файл к модулю 
//****************************************************************************** 
#include  "RTC_IT.h"      // Включаем файл заголовка для нашего модуля

extern uint32_t vectorTable_RAM[];
void (*pfUserISR)(void) = NULL;

RTC_main::RTC_main(uint8_t RTC_Tact)
{
  _RTC_FreqSource = RTC_Tact;
  _Alarm = 0;
}

uint8_t RTC_main::Reinit(void)
{
  RCC->APB1ENR |= RCC_APB1ENR_PWREN;                                // Включить тактирование PWR и Backup
  RCC->AHB1ENR |= RCC_AHB1ENR_BKPSRAMEN;
  PWR->CR |= PWR_CR_DBP;                                            // Разрешить доступ к Backup области
  RCC->BDCR |= RCC_BDCR_BDRST;                                      // Сбросить Backup область
  RCC->BDCR &= ~RCC_BDCR_BDRST;
  PWR->CR &= ~PWR_CR_DBP;                                           // запретить доступ к Backup области
  return init();
}

uint8_t RTC_main::init(void)
{
  uint32_t StartUpCounter;

  if((RCC->BDCR & RCC_BDCR_RTCEN) != RCC_BDCR_RTCEN)                  // Проверка работы часов, если не включены, то инициализировать
  {
    RCC->APB1ENR |= RCC_APB1ENR_PWREN;                                // Включить тактирование PWR и Backup
    RCC->AHB1ENR |= RCC_AHB1ENR_BKPSRAMEN;
    PWR->CR |= PWR_CR_DBP;                                            // Разрешить доступ к Backup области
    RCC->BDCR |= RCC_BDCR_BDRST;                                      // Сбросить Backup область
    RCC->BDCR &= ~RCC_BDCR_BDRST;
//
    switch (_RTC_FreqSource)
    {
      // ---------- LSI ------------ //
      case RTC_LSI:                                                   // Внутренний генератор 32 000 Гц
        _Prediv_A = 0x7C;                                             // Предделитель RTC
        _Prediv_S = 0xFF;
        RCC->CSR |= RCC_CSR_LSION;                                    // Включаем LSI
        for (StartUpCounter = 0;; StartUpCounter++)                   // В цикле ждём определённое время ожидая запуска
        {
          if(RCC->CSR & RCC_CSR_LSIRDY) break;                        // Запуск есть, выходим из цикла
          if(StartUpCounter > 0x5000)
          {
            RCC->CR &= ~RCC_CSR_LSION;                                // Отключаем LSI
            RCC->BDCR &= 0x00 << RCC_BDCR_RTCSEL_Pos;                 // Сросить выбор тактирования
            RCC->BDCR &= ~RCC_BDCR_RTCEN;                             // Отключить тактирование
            return false;                                             // Запуска нет, выходим из функции
          }
        }
        RCC->BDCR |= _RTC_FreqSource << RCC_BDCR_RTCSEL_Pos;              // Выбрать источник
        RCC->BDCR |= RCC_BDCR_RTCEN;                                      // Подать тактирование ??????????????????
        break;
      // ----------- LSE ----------- //
      case RTC_LSE:                                                   // Кварцевый генератор 32 768 Гц
        RCC->BDCR |= RCC_BDCR_RTCSEL_0;                                   // Выбрать LSE источник (кварц 32768)
        RCC->BDCR |= RCC_BDCR_RTCEN;                                      // Подать тактирование
        RCC->BDCR |= RCC_BDCR_LSEON;                                      // Включить LSE
        for (StartUpCounter = 0;; StartUpCounter++)                   // В цикле ждём определённое время ожидая запуска
        {
          if(RCC->BDCR & RCC_BDCR_LSERDY) break;                      // Запуск есть, выходим из цикла
          if(StartUpCounter > 0x500000)
          {
            RCC->CR &= ~RCC_BDCR_LSEON;                               // Отключаем LSE
            RCC->BDCR &= 0x00 << RCC_BDCR_RTCSEL_Pos;                 // Сбросить выбор тактирования
            RCC->BDCR &= ~RCC_BDCR_RTCEN;                             // Отключить тактирование
            return false;                                             // Запуска нет, выходим из функции
          }
        }
        _Prediv_A = 0x7F;                                             // Делитель для кварца
        _Prediv_S = 0xFF;
        break;
      // ----------- HSE ----------- //
      case RTC_HSE:                                                   // Основной кварцевый резонатор
        break;
      default:
        break;
    }
    RTC->WPR = 0xCA;                                                  // Выключаем защиту от записи
    RTC->WPR = 0x53;
    RTC->ISR |= RTC_ISR_INIT;                                         // Входим в режим редактирования
    for (StartUpCounter = 0;; StartUpCounter++)                       // В цикле ждём определённое время ожидая запуска
    {
      if(RTC->ISR & RTC_ISR_INITF) break;                             // Инициализация разрешена, выходим из цикла
      if(StartUpCounter > 0x500000)
      {
        RCC->CR &= ~RCC_CSR_LSION;                                    // Отключаем LSI
        RCC->BDCR &= ~RCC_BDCR_LSEON;                                 // Отключаем LSE
        RCC->BDCR &= 0x00 << RCC_BDCR_RTCSEL_Pos;                     // Отключаем выбор источника
        PWR->CR &= ~PWR_CR_DBP;                                       // Запретить доступ к Backup области
        RTC->WPR = 0xFF;                                              // Включаем защиту от записи
        RCC->BDCR &= ~RCC_BDCR_RTCEN;                                 // Отключить тактирование
        return false;                                                 // Запуска нет, выходим из функции
      }
    }

    RTC->PRER = (RTC->PRER & (~(0x7F << RTC_PRER_PREDIV_A_Pos))) | (_Prediv_A << RTC_PRER_PREDIV_A_Pos);
    RTC->PRER = (RTC->PRER & (~(0x7FFF << RTC_PRER_PREDIV_S_Pos))) | (_Prediv_S << RTC_PRER_PREDIV_S_Pos);
    RTC->ISR &= ~(RTC_ISR_INIT);                                      // Выходим из режима редактирования
    RTC->WPR = 0xFF;                                                  // Включаем защиту от записи
    PWR->CR &= ~PWR_CR_DBP;                                           // запретить доступ к Backup области
    for (StartUpCounter = 0;; StartUpCounter++)                       // В цикле ждём определённое время ожидая отключения
    {
      if(RTC->ISR & RTC_ISR_RSF) break;                            // Ждём окончания синхронизации
      if(StartUpCounter > 0x500000)
      {
        RCC->CR &= ~RCC_CSR_LSION;                                    // Отключаем LSI
        RCC->BDCR &= ~RCC_BDCR_LSEON;
        RCC->BDCR &= 0x00 << RCC_BDCR_RTCSEL_Pos;                     // Отключаем выбор источника
        PWR->CR &= ~PWR_CR_DBP;                                       // Запретить доступ к Backup области
        RTC->WPR = 0xFF;                                              // Включаем защиту от записи
        RCC->BDCR &= ~RCC_BDCR_RTCEN;                                 // Отключить тактирование
        return false;                                                 // Запуска нет, выходим из функции
      }
    }
  }
  return _RTC_FreqSource;
}


/*
 * Инициализация прерывания периодического счётчика
 * Счётчик считает секундные импульсы.
 * Timer - Количество секунд
 */
void RTC_main::setIT_WUCK(callback_IRQ ExternIT, uint16_t Timer)
{
  if(RCC->BDCR & RCC_BDCR_RTCEN)                  // Проверка работы часов, если не включены, то не инициализировать
  {
    // Устанавливаем прерывание
    EXTI->IMR |= EXTI_IMR_MR22;
    EXTI->RTSR |= EXTI_RTSR_TR22;

    PWR->CR |= PWR_CR_DBP;                                            // Разрешить доступ к Backup области
    RTC->WPR = 0xCA;                                                  // Выключаем защиту от записи
    RTC->WPR = 0x53;

    // Выключаем таймер
    RTC->CR &= ~RTC_CR_WUTE;
    while (!(RTC->ISR & RTC_ISR_WUTWF)){}
    RTC->ISR &= ~RTC_ISR_WUTF;
    RTC->CR |= RTC_CR_WUTIE;
    // Ставим делитель на нужный режим
    RTC->WUTR = Timer - 1; //2048; //4096;
    RTC->CR &= ~RTC_CR_WUCKSEL;                                         // Сбрасываем биты, получаем деление на 16
    RTC->CR |= RTC_CR_WUCKSEL_2;                                      // Устанавливаем 3й бит, получаем секундные импульсы на входе счётчика

    RTC->CR |= RTC_CR_WUTE;                                           // Запускаем
    while (RTC->ISR & RTC_ISR_WUTWF){}

    RTC->WPR = 0xFF;                                                  // Включаем защиту от записи
    PWR->CR &= ~PWR_CR_DBP;                                           // запретить доступ к Backup области

    __disable_irq();
    vectorTable_RAM[RTC_WKUP_IRQn + 16] = (uint32_t)myISR_WKUP;       // Подменяем на свой вектор прерывания
    __enable_irq();

    pfUserISR = ExternIT;                                             // Ссылка на наш, внешний обработчик прерывания

    NVIC_EnableIRQ(RTC_WKUP_IRQn);
  }
}

/*
 * Установка даты.
 * Формат: setDate( ДД, ММ, ГГГГ, Н );
 *
 */
void RTC_main::setDate(uint8_t Date, uint8_t Month, uint16_t Year, uint8_t Week )
{
  uint16_t tempYear = Year%100;

  if(RCC->BDCR & RCC_BDCR_RTCEN)                  // Проверка работы часов, если не включены, то не инициализировать
  {
    PWR->CR |= PWR_CR_DBP;                                              // Разрешить доступ к Backup области
    RTC->WPR = 0xCA;                                                    // Выключаем защиту от записи
    RTC->WPR = 0x53;
    RTC->ISR |= RTC_ISR_INIT;                                           // Входим в режим редактирования календаря
    while (!(RTC->ISR & RTC_ISR_INITF)){}                               // Ждем подтверждения входа в режим редактирования
    RTC->DR = ((tempYear / 10) << RTC_DR_YT_Pos)
        | ((tempYear % 10) << RTC_DR_YU_Pos)
        | (Week << RTC_DR_WDU_Pos)
        | ((Month / 10) << RTC_DR_MT_Pos)
        | ((Month % 10) << RTC_DR_MU_Pos)
        | ((Date / 10) << RTC_DR_DT_Pos)
        | ((Date % 10) << RTC_DR_DU_Pos);
    RTC->ISR &= ~(RTC_ISR_INIT);                                        // Выходим из режима редактирования
    RTC->WPR = 0xFF;                                                    // Включаем защиту от записи
    PWR->CR &= ~PWR_CR_DBP;                                             // запретить доступ к Backup области
  }
}

/*
 * Установка времени
 * Формат: setTime( ЧЧ, ММ, СС );
 */
void RTC_main::setTime(uint8_t Hour, uint8_t Minute, uint8_t Sec)
{
  if(RCC->BDCR & RCC_BDCR_RTCEN)                  // Проверка работы часов, если не включены, то не инициализировать
  {
    PWR->CR |= PWR_CR_DBP;                                              // Разрешить доступ к Backup области
    RTC->WPR = 0xCA;                                                    // Выключаем защиту от записи
    RTC->WPR = 0x53;
    RTC->ISR |= RTC_ISR_INIT;                                           // Входим в режим редактирования календаря
    while (!(RTC->ISR & RTC_ISR_INITF)){};                              // Ждем подтверждения входа в режим редактирования
    RTC->TR = ((Hour/10) << RTC_TR_HT_Pos)    |
              ((Hour%10) << RTC_TR_HU_Pos)    |
              ((Minute/10) << RTC_TR_MNT_Pos) |
              ((Minute%10) << RTC_TR_MNU_Pos) |
              ((Sec/10) << RTC_TR_ST_Pos)     |
              ((Sec%10) << RTC_TR_SU_Pos);
    RTC->ISR &= ~(RTC_ISR_INIT);                                        // Выходим из режима редактирования
    RTC->WPR = 0xFF;                                                    // Включаем защиту от записи
    PWR->CR &= ~PWR_CR_DBP;                                             // запретить доступ к Backup области
  }
}

// Вернуть год
uint16_t  RTC_main::getYear(void)
{
  return ((RTC->DR & RTC_DR_YT) >> RTC_DR_YT_Pos) *10 + ((RTC->DR & RTC_DR_YU) >> RTC_DR_YU_Pos) + 2000;
}

// Вернуть день недели
uint8_t   RTC_main::getWeek(void)
{
  return ((RTC->DR & RTC_DR_WDU) >> RTC_DR_WDU_Pos);
}

// Вернуть месяц
uint8_t   RTC_main::getMonth(void)
{
  return ((RTC->DR & RTC_DR_MT) >> RTC_DR_MT_Pos) *10 + ((RTC->DR & RTC_DR_MU) >> RTC_DR_MU_Pos);
}

// Вернуть дату
uint8_t   RTC_main::getDate(void)
{
  return ((RTC->DR & RTC_DR_DT) >> RTC_DR_DT_Pos) *10 + ((RTC->DR & RTC_DR_DU) >> RTC_DR_DU_Pos);
}

// Вернуть час
uint8_t   RTC_main::getHour(void)
{
  return ((RTC->TR & RTC_TR_HT) >> RTC_TR_HT_Pos) *10 + ((RTC->TR & RTC_TR_HU) >> RTC_TR_HU_Pos);
}

// Вернуть минуты
uint8_t   RTC_main::getMinute(void)
{
  return ((RTC->TR & RTC_TR_MNT) >> RTC_TR_MNT_Pos) *10 + ((RTC->TR & RTC_TR_MNU) >> RTC_TR_MNU_Pos);
}

// Вернуть секунды
uint8_t   RTC_main::getSeconds(void)
{
  return ((RTC->TR & RTC_TR_ST) >> RTC_TR_ST_Pos) *10 + ((RTC->TR & RTC_TR_SU) >> RTC_TR_SU_Pos);
}


void RTC_main::myISR_WKUP(void)
{
  PWR->CR   |= PWR_CR_DBP;                                            // Разрешить доступ к Backup области
  RTC->ISR  &= ~RTC_ISR_WUTF;
  EXTI->PR  |= EXTI_PR_PR22;
  PWR->CR   &= ~PWR_CR_DBP;                                           // запретить доступ к Backup области

  if(pfUserISR != NULL) pfUserISR();
}


//****************************************************************************** 
//  ENF OF FILE 
//****************************************************************************** 

